r"""
Derivations

Let `A` be a ring and `B` be a bimodule over `A`.
A derivation `d : A \to B` is an additive map that satisfies
the Leibniz rule

.. MATH::

    d(xy) = x d(y) + d(x) y.

If `B` is an algebra over `A` and if we are given in addition a
ring homomorphism `\theta : A \to B`, a twisted derivation with respect
to `\theta` (or a `\theta`-derivation) is an additive map `d : A \to B`
such that

.. MATH::

    d(xy) = \theta(x) d(y) + d(x) y.

When `\theta` is the morphism defining the structure of `A`-algebra
on `B`, a `\theta`-derivation is nothing but a derivation.
In general, if `\iota : A \to B` denotes the defining morphism above,
one easily checks that `\theta - \iota` is a `\theta`-derivation.

This file provides support for derivations and twisted derivations
over commutative rings with values in algebras (i.e. we require
that `B` is a commutative `A`-algebra).
In this case, the set of derivations (resp. `\theta`-derivations)
is a module over `B`.

Given a ring `A`, the module of derivations over `A` can be created
as follows::

    sage: A.<x,y,z> = QQ[]
    sage: M = A.derivation_module()
    sage: M
    Module of derivations over
     Multivariate Polynomial Ring in x, y, z over Rational Field

The method :meth:`~sage.rings.derivation.RingDerivationModule.gens`
returns the generators of this module::

    sage: A.<x,y,z> = QQ[]
    sage: M = A.derivation_module()
    sage: M.gens()
    (d/dx, d/dy, d/dz)

We can combine them in order to create all derivations::

    sage: d = 2*M.gen(0) + z*M.gen(1) + (x^2 + y^2)*M.gen(2)
    sage: d
    2*d/dx + z*d/dy + (x^2 + y^2)*d/dz

and now play with them::

    sage: d(x + y + z)
    x^2 + y^2 + z + 2
    sage: P = A.random_element()
    sage: Q = A.random_element()
    sage: d(P*Q) == P*d(Q) + d(P)*Q
    True

Alternatively we can use the method
:meth:`~sage.categories.commutative_rings.CommutativeRings.ParentMethods.derivation`
of the ring `A` to create derivations::

    sage: Dx = A.derivation(x); Dx
    d/dx
    sage: Dy = A.derivation(y); Dy
    d/dy
    sage: Dz = A.derivation(z); Dz
    d/dz
    sage: A.derivation([2, z, x^2+y^2])
    2*d/dx + z*d/dy + (x^2 + y^2)*d/dz

Sage knows moreover that `M` is a Lie algebra::

    sage: M.category()
    Join of
     Category of Lie algebras with basis over Rational Field and
     Category of modules with basis over
      Multivariate Polynomial Ring in x, y, z over Rational Field

Computations of Lie brackets are implemented as well::

    sage: Dx.bracket(Dy)
    0
    sage: d.bracket(Dx)
    -2*x*d/dz

At the creation of a module of derivations, a codomain can be specified::

    sage: B = A.fraction_field()
    sage: A.derivation_module(B)
    Module of derivations from Multivariate Polynomial Ring in x, y, z over Rational Field
     to Fraction Field of Multivariate Polynomial Ring in x, y, z over Rational Field

Alternatively, one can specify a morphism `f` with domain `A`.
In this case, the codomain of the derivations is the codomain of
`f` but the latter is viewed as an algebra over `A` through the
homomorphism `f`.
This construction is useful, for example, if we want to work with
derivations on `A` at a certain point, e.g. `(0,1,2)`. Indeed,
in order to achieve this, we first define the evaluation map at
this point::

    sage: ev = A.hom([QQ(0), QQ(1), QQ(2)])
    sage: ev
    Ring morphism:
      From: Multivariate Polynomial Ring in x, y, z over Rational Field
      To:   Rational Field
      Defn: x |--> 0
            y |--> 1
            z |--> 2

Now we use this ring homomorphism to define a structure of `A`-algebra
on `\QQ` and then build the following module of derivations::

    sage: M = A.derivation_module(ev)
    sage: M
    Module of derivations
     from Multivariate Polynomial Ring in x, y, z over Rational Field
       to Rational Field
    sage: M.gens()
    (d/dx, d/dy, d/dz)

Elements in `M` then acts as derivations at `(0,1,2)`::

    sage: Dx = M.gen(0)
    sage: Dy = M.gen(1)
    sage: Dz = M.gen(2)
    sage: f = x^2 + y^2 + z^2
    sage: Dx(f)  # = 2*x evaluated at (0,1,2)
    0
    sage: Dy(f)  # = 2*y evaluated at (0,1,2)
    2
    sage: Dz(f)  # = 2*z evaluated at (0,1,2)
    4


Twisted derivations are handled similarly::

    sage: theta = B.hom([B(y),B(z),B(x)])
    sage: theta
    Ring endomorphism of Fraction Field of
     Multivariate Polynomial Ring in x, y, z over Rational Field
      Defn: x |--> y
            y |--> z
            z |--> x

    sage: M = B.derivation_module(twist=theta)
    sage: M
    Module of twisted derivations over Fraction Field of Multivariate Polynomial Ring
     in x, y, z over Rational Field (twisting morphism: x |--> y, y |--> z, z |--> x)

Over a field, one proves that every `\theta`-derivation is a multiple
of `\theta - id`, so that::

    sage: d = M.gen(); d
    [x |--> y, y |--> z, z |--> x] - id

and then::

    sage: d(x)
    -x + y
    sage: d(y)
    -y + z
    sage: d(z)
    x - z
    sage: d(x + y + z)
    0

AUTHOR:

- Xavier Caruso (2018-09)
"""

# ***************************************************************************
#    Copyright (C) 2018 Xavier Caruso <xavier.caruso@normalesup.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 2 of the License, or
#    (at your option) any later version.
#                  http://www.gnu.org/licenses/
# ***************************************************************************

from sage.structure.richcmp import op_EQ, op_NE
from sage.structure.unique_representation import UniqueRepresentation
from sage.sets.family import Family
from sage.modules.module import Module
from sage.structure.element import ModuleElement
from sage.rings.integer_ring import ZZ

from sage.rings.polynomial.polynomial_ring import PolynomialRing_generic
from sage.rings.polynomial.multi_polynomial_ring_base import MPolynomialRing_base
from sage.rings.power_series_ring import PowerSeriesRing_generic
from sage.rings.laurent_series_ring import LaurentSeriesRing
from sage.rings.fraction_field import FractionField_generic
from sage.rings.quotient_ring import QuotientRing_generic
from sage.rings.polynomial.polynomial_quotient_ring import PolynomialQuotientRing_generic
from sage.rings.finite_rings.integer_mod_ring import IntegerModRing_generic
from sage.rings.padics.padic_generic import pAdicGeneric
from sage.rings.function_field.function_field import FunctionField
from sage.rings.function_field.function_field_rational import RationalFunctionField
from sage.categories.number_fields import NumberFields
from sage.categories.finite_fields import FiniteFields
from sage.categories.modules import Modules
from sage.categories.modules_with_basis import ModulesWithBasis
from sage.categories.lie_algebras import LieAlgebras

from sage.categories.map import Map
from sage.categories.rings import Rings

from sage.misc.latex import latex


class RingDerivationModule(Module, UniqueRepresentation):
    """
    A class for modules of derivations over a commutative ring.
    """
    def __init__(self, domain, codomain, twist=None):
        """
        Initialize this module of derivation.

        TESTS::

            sage: A.<x,y> = QQ[]
            sage: M = A.derivation_module()
            sage: TestSuite(M).run()

            sage: from sage.rings.derivation import RingDerivationModule
            sage: R5.<x> = GF(5)[]
            sage: R25.<x> = GF(25)[]                                                    # needs sage.rings.finite_rings
            sage: R7.<x> = GF(7)[]

            sage: RingDerivationModule(R5, R25)                                         # needs sage.rings.finite_rings
            Module of derivations
             from Univariate Polynomial Ring in x over Finite Field of size 5
               to Univariate Polynomial Ring in x over Finite Field in z2 of size 5^2
            sage: RingDerivationModule(R5, R5^2)
            Traceback (most recent call last):
            ...
            TypeError: the codomain must be an algebra over the domain
             or a morphism with the correct domain
            sage: RingDerivationModule(R5, R7)                                          # needs sage.rings.finite_rings
            Traceback (most recent call last):
            ...
            TypeError: the codomain must be an algebra over the domain
             or a morphism with the correct domain

            sage: theta = R5.hom([R5.gen()^2])
            sage: RingDerivationModule(R5, R25, twist=theta)                            # needs sage.rings.finite_rings
            Module of twisted derivations
             from Univariate Polynomial Ring in x over Finite Field of size 5
               to Univariate Polynomial Ring in x over Finite Field in z2 of size 5^2
             (twisting morphism: x |--> x^2)
            sage: RingDerivationModule(R7, R7, twist=theta)                             # needs sage.rings.finite_rings
            Traceback (most recent call last):
            ...
            TypeError: the domain of the derivation must coerce to the domain of the twisting homomorphism
        """
        if domain not in Rings().Commutative():
            raise TypeError("the domain must be a commutative ring")

        if codomain in Rings().Commutative() and codomain.has_coerce_map_from(domain):
            defining_morphism = codomain.coerce_map_from(domain)
        elif (isinstance(codomain,Map)
              and codomain.category_for().is_subcategory(Rings())
              and codomain.domain().has_coerce_map_from(domain)):
            if codomain.domain() is domain:
                defining_morphism = codomain
            else:
                defining_morphism = codomain * codomain.domain().coerce_map_from(domain)
            codomain = defining_morphism.codomain()
        else:
            raise TypeError("the codomain must be an algebra over the domain"
                            " or a morphism with the correct domain")

        if twist is not None:
            if not (isinstance(twist, Map) and twist.category_for().is_subcategory(Rings())):
                raise TypeError("the twisting homomorphism must be a homomorphism of rings")
            if twist.domain() is not domain:
                map = twist.domain().coerce_map_from(domain)
                if map is None:
                    raise TypeError("the domain of the derivation must coerce"
                                    " to the domain of the twisting homomorphism")
                twist = twist * map
            if twist.codomain() is not codomain:
                map = codomain.coerce_map_from(twist.codomain())
                if map is None:
                    raise TypeError("the codomain of the twisting homomorphism"
                                    " must coerce to the codomain of the derivation")
                twist = map * twist
            # We check if the twisting morphism is the defining morphism
            try:
                if twist == defining_morphism:
                    twist = None
                else:
                    for g in domain.gens():
                        if twist(g) != defining_morphism(g):
                            break
                    else:
                        twist = None
            except (AttributeError, NotImplementedError):
                pass

        self._domain = domain
        self._codomain = codomain
        self._defining_morphism = defining_morphism
        self._twist = twist
        self._base_derivation = None
        self._gens = None
        self._basis = self._dual_basis = None
        # Currently basis and gens play exactly the same role because
        # the only rings that are supported lead to free modules of derivations
        # So the code is a bit redundant but we except to be able to cover more
        # rings (with non free modules of derivations) in a near future
        self._constants = (ZZ, False)
        if twist is not None:
            self.Element = RingDerivationWithTwist_generic
            if domain.is_field():
                self._gens = [ 1 ]
                self._basis = [ 1 ]
        elif (domain is ZZ or domain in NumberFields() or domain in FiniteFields()
              or isinstance(domain, IntegerModRing_generic)
              or (isinstance(domain, pAdicGeneric) and (domain.is_field() or domain.absolute_e() == 1))):
            self.Element = RingDerivationWithoutTwist_zero
            self._gens = [ ]
            self._basis = [ ]
            self._dual_basis = [ ]
            self._constants = (domain, True)
        elif (isinstance(domain, (PolynomialRing_generic, MPolynomialRing_base, PowerSeriesRing_generic, LaurentSeriesRing))
              or (isinstance(domain, FractionField_generic)
                  and isinstance(domain.ring(), (PolynomialRing_generic, MPolynomialRing_base)))):
            self._base_derivation = RingDerivationModule(domain.base_ring(), defining_morphism)
            self.Element = RingDerivationWithoutTwist_function
            try:
                self._gens = self._base_derivation.gens() + domain.gens()
            except NotImplementedError:
                pass
            try:
                self._basis = tuple(self._base_derivation.basis()) + domain.gens()
                self._dual_basis = tuple(self._base_derivation.dual_basis()) + domain.gens()
            except NotImplementedError:
                pass
            constants, sharp = self._base_derivation._constants
            if domain.characteristic() == 0:
                self._constants = (constants, sharp)
            else:
                # in this case, the constants are polynomials in x^p
                # TODO: implement this
                self._constants = (constants, False)
        elif isinstance(domain, FractionField_generic):
            self._base_derivation = RingDerivationModule(domain.ring(), defining_morphism)
            self.Element = RingDerivationWithoutTwist_fraction_field
            try:
                self._gens = self._base_derivation.gens()
            except NotImplementedError:
                pass
            try:
                self._basis = self._base_derivation.basis()
                self._dual_basis = self._base_derivation.dual_basis()
            except NotImplementedError:
                pass
            constants, sharp = self._base_derivation._constants
            self._constants = (constants.fraction_field(), False)
        elif isinstance(domain, PolynomialQuotientRing_generic):
            self._base_derivation = RingDerivationModule(domain.base(), defining_morphism)
            modulus = domain.modulus()
            for der in self._base_derivation.gens():
                if der(modulus) != 0:
                    raise NotImplementedError("derivations over quotient rings"
                                              " are not fully supported")
            self.Element = RingDerivationWithoutTwist_quotient
            try:
                self._gens = self._base_derivation.gens()
            except NotImplementedError:
                pass
            try:
                self._basis = self._base_derivation.basis()
                self._dual_basis = self._base_derivation.dual_basis()
            except NotImplementedError:
                pass
            constants, sharp = self._base_derivation._constants
            self._constants = (constants, False)  # can we do better?
        elif isinstance(domain, QuotientRing_generic):
            self._base_derivation = RingDerivationModule(domain.cover_ring(), defining_morphism)
            if any(der(modulus) != 0 for modulus in domain.defining_ideal().gens()
                   for der in self._base_derivation.gens()):
                raise NotImplementedError("derivations over quotient rings"
                                          " are not fully supported")
            self.Element = RingDerivationWithoutTwist_quotient
            try:
                self._gens = self._base_derivation.gens()
            except NotImplementedError:
                pass
            try:
                self._basis = self._base_derivation.basis()
                self._dual_basis = self._base_derivation.dual_basis()
            except NotImplementedError:
                pass
            constants, sharp = self._base_derivation._constants
            self._constants = (constants, False)  # can we do better?
        elif isinstance(domain, RationalFunctionField):
            from sage.rings.function_field.derivations_rational import FunctionFieldDerivation_rational
            self.Element = FunctionFieldDerivation_rational
            self._gens = self._basis = [ None ]
            self._dual_basis = [ domain.gen() ]
        elif isinstance(domain, FunctionField):
            if domain.is_separable():
                from sage.rings.function_field.derivations_polymod import FunctionFieldDerivation_separable
                self._base_derivation = RingDerivationModule(domain.base_ring(), defining_morphism)
                self.Element = FunctionFieldDerivation_separable
                try:
                    self._gens = self._base_derivation.gens()
                except NotImplementedError:
                    pass
                try:
                    self._basis = self._base_derivation.basis()
                    self._dual_basis = self._base_derivation.dual_basis()
                except NotImplementedError:
                    pass
            else:
                from sage.rings.function_field.derivations_polymod import FunctionFieldDerivation_inseparable
                M, f, self._t = domain.separable_model()
                self._base_derivation = RingDerivationModule(M, defining_morphism * f)
                self._d = self._base_derivation(None)
                self.Element = FunctionFieldDerivation_inseparable
                self._gens = self._basis = [ None ]
                self._dual_basis = [ f(M.base_ring().gen()) ]
        else:
            raise NotImplementedError("derivations over this ring is not implemented")
        if self._basis is None:
            category = Modules(codomain)
        else:
            category = ModulesWithBasis(codomain)
        if self._twist is None and domain is codomain:
            category &= LieAlgebras(self._constants[0])
        Module.__init__(self, codomain, category=category)
        if self._gens is not None:
            self._gens = [self.element_class(self, x) for x in self._gens]
        if self._basis is not None:
            self._basis = [self.element_class(self, x) for x in self._basis]
        if self._dual_basis is not None:
            self._dual_basis = [domain(x) for x in self._dual_basis]

    def __hash__(self):
        """
        Return a hash of ``self``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module()
            sage: hash(M) == hash((M.domain(), M.codomain(), M.twisting_morphism()))
            True
        """
        return hash((self._domain, self._codomain, self._twist))

    def _coerce_map_from_(self, R):
        """
        Return ``True`` if there is a coercion map from ``R``
        to this module.

        EXAMPLES::

            sage: A.<x> = QQ[]
            sage: B.<y> = A[]
            sage: M1 = A.derivation_module(); M1
            Module of derivations over Univariate Polynomial Ring in x over Rational Field
            sage: M2 = A.derivation_module(B); M2
            Module of derivations
             from Univariate Polynomial Ring in x over Rational Field
               to Univariate Polynomial Ring in y over
                   Univariate Polynomial Ring in x over Rational Field
            sage: M1._coerce_map_from_(M2) is None
            True
            sage: M1.has_coerce_map_from(M2)
            False
            sage: M2.has_coerce_map_from(M1)
            True

            sage: M1.has_coerce_map_from(ZZ)
            False
            sage: M1.has_coerce_map_from(QQ)
            False
            sage: M1.has_coerce_map_from(A)
            False
        """
        if isinstance(R, RingDerivationModule):
            if R.domain().has_coerce_map_from(self._domain) and self._codomain.has_coerce_map_from(R.codomain()):
                morR = R.defining_morphism()
                morS = self._defining_morphism
                try:
                    # this test is not perfect
                    return all(morR(g) == morS(g) for g in self._domain.gens())
                except (AttributeError, NotImplementedError):
                    pass
        return super()._coerce_map_from_(R)

    def _repr_(self):
        """
        Return a string representation of this module of derivations.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: R.derivation_module()
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring

            sage: theta = R.hom([y,x])
            sage: R.derivation_module(twist=theta)
            Module of twisted derivations over Multivariate Polynomial Ring in x, y
             over Integer Ring (twisting morphism: x |--> y, y |--> x)
        """
        t = ""
        if self._twist is None:
            s = "Module of derivations"
        else:
            s = "Module of twisted derivations"
            try:
                t = " (twisting morphism: %s)" % self._twist._repr_short()
            except AttributeError:
                pass
        if self._domain is self._codomain:
            s += " over %s" % self._domain
        else:
            s += " from %s to %s" % (self._domain, self._codomain)
        return s + t

    def domain(self):
        """
        Return the domain of the derivations in this module.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module(); M
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring
            sage: M.domain()
            Multivariate Polynomial Ring in x, y over Integer Ring
        """
        return self._domain

    def codomain(self):
        """
        Return the codomain of the derivations in this module.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module(); M
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring
            sage: M.codomain()
            Multivariate Polynomial Ring in x, y over Integer Ring
        """
        return self._codomain

    def defining_morphism(self):
        """
        Return the morphism defining the structure of algebra
        of the codomain over the domain.

        EXAMPLES::

            sage: R.<x> = QQ[]
            sage: M = R.derivation_module()
            sage: M.defining_morphism()
            Identity endomorphism of Univariate Polynomial Ring in x over Rational Field

            sage: S.<y> = R[]
            sage: M = R.derivation_module(S)
            sage: M.defining_morphism()
            Polynomial base injection morphism:
              From: Univariate Polynomial Ring in x over Rational Field
              To:   Univariate Polynomial Ring in y over
                     Univariate Polynomial Ring in x over Rational Field

            sage: ev = R.hom([QQ(0)])
            sage: M = R.derivation_module(ev)
            sage: M.defining_morphism()
            Ring morphism:
              From: Univariate Polynomial Ring in x over Rational Field
              To:   Rational Field
              Defn: x |--> 0
        """
        return self._defining_morphism

    def twisting_morphism(self):
        r"""
        Return the twisting homomorphism of the derivations in this module.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: M = R.derivation_module(twist=theta); M
            Module of twisted derivations over Multivariate Polynomial Ring in x, y
             over Integer Ring (twisting morphism: x |--> y, y |--> x)
            sage: M.twisting_morphism()
            Ring endomorphism of Multivariate Polynomial Ring in x, y over Integer Ring
              Defn: x |--> y
                    y |--> x

        When the derivations are untwisted, this method returns nothing::

            sage: M = R.derivation_module()
            sage: M.twisting_morphism()
        """
        return self._twist

    def ngens(self):
        r"""
        Return the number of generators of this module of derivations.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module(); M
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring
            sage: M.ngens()
            2

        Indeed, generators are::

            sage: M.gens()
            (d/dx, d/dy)

        We check that, for a nontrivial twist over a field, the module of
        twisted derivation is a vector space of dimension 1 generated by
        ``twist - id``::

            sage: K = R.fraction_field()
            sage: theta = K.hom([K(y),K(x)])
            sage: M = K.derivation_module(twist=theta); M
            Module of twisted derivations over Fraction Field of Multivariate Polynomial
             Ring in x, y over Integer Ring (twisting morphism: x |--> y, y |--> x)
            sage: M.ngens()
            1
            sage: M.gen()
            [x |--> y, y |--> x] - id
        """
        if self._gens is None:
            raise NotImplementedError("generators are not implemented for this derivation module")
        return len(self._gens)

    def gens(self) -> tuple:
        r"""
        Return the generators of this module of derivations.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module(); M
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring
            sage: M.gens()
            (d/dx, d/dy)

        We check that, for a nontrivial twist over a field, the module of
        twisted derivation is a vector space of dimension 1 generated by
        ``twist - id``::

            sage: K = R.fraction_field()
            sage: theta = K.hom([K(y),K(x)])
            sage: M = K.derivation_module(twist=theta); M
            Module of twisted derivations over Fraction Field of Multivariate Polynomial
             Ring in x, y over Integer Ring (twisting morphism: x |--> y, y |--> x)
            sage: M.gens()
            ([x |--> y, y |--> x] - id,)
        """
        if self._gens is None:
            raise NotImplementedError("generators are not implemented for this derivation module")
        return tuple(self._gens)

    def gen(self, n=0):
        r"""
        Return the `n`-th generator of this module of derivations.

        INPUT:

        - ``n`` -- integer (default: `0`)

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module(); M
            Module of derivations over Multivariate Polynomial Ring in x, y over Integer Ring
            sage: M.gen()
            d/dx
            sage: M.gen(1)
            d/dy
        """
        if self._gens is None:
            raise NotImplementedError("generators are not implemented for this derivation module")
        try:
            return self._gens[n]
        except IndexError:
            raise ValueError("generator not defined")

    def basis(self):
        r"""
        Return a basis of this module of derivations.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)
        """
        if self._basis is None:
            raise NotImplementedError("basis is not implemented for this derivation module")
        return Family(self._basis)

    def dual_basis(self):
        r"""
        Return the dual basis of the canonical basis of this module of
        derivations (which is that returned by the method :meth:`basis`).

        .. NOTE::

            The dual basis of `(d_1, \dots, d_n)` is a family
            `(x_1, \ldots, x_n)` of elements in the domain such
            that `d_i(x_i) = 1` and `d_i(x_j) = 0` if `i \neq j`.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)
            sage: M.dual_basis()
            Family (x, y)
        """
        if self._dual_basis is None:
            raise NotImplementedError("basis is not implemented for this derivation module")
        return Family(self._dual_basis)

    def ring_of_constants(self):
        r"""
        Return the subring of the domain consisting of elements
        `x` such that `d(x) = 0` for all derivation `d` in this module.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)
            sage: M.ring_of_constants()
            Rational Field
        """
        if not self._constants[1]:
            raise NotImplementedError("the computation of the ring of constants"
                                      " is not implemented for this derivation module")
        return self._constants[0]

    def random_element(self, *args, **kwds):
        r"""
        Return a random derivation in this module.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module()
            sage: M.random_element()  # random
            (x^2 + x*y - 3*y^2 + x + 1)*d/dx + (-2*x^2 + 3*x*y + 10*y^2 + 2*x + 8)*d/dy
        """
        if self._gens is None:
            raise NotImplementedError("generators are not implemented for this derivation module")
        return self([ self._codomain.random_element(*args, **kwds) for _ in range(len(self._gens)) ])

    def some_elements(self):
        r"""
        Return a list of elements of this module.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: M = R.derivation_module()
            sage: M.some_elements()
            [d/dx, d/dy, x*d/dx, x*d/dy, y*d/dx, y*d/dy]
        """
        if self._gens is None:
            return self.an_element()
        if self._dual_basis is None:
            return self._gens
        return self._gens + [f * D for f in self._dual_basis for D in self._gens]


# The class RingDerivation does not derive from Map (or RingMap)
# because we don't want to see derivations as morphisms in some
# category since they are not stable by composition.
class RingDerivation(ModuleElement):
    r"""
    An abstract class for twisted and untwisted derivations over
    commutative rings.

    TESTS::

        sage: R.<x,y> = ZZ[]
        sage: f = R.derivation(x) + 2*R.derivation(y); f
        d/dx + 2*d/dy
        sage: f(x*y)
        2*x + y
    """
    def __call__(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = x*R.derivation(x) + y*R.derivation(y)
            sage: f(x^2 + 3*x*y - y^2)
            2*x^2 + 6*x*y - 2*y^2
        """
        arg = self.parent().domain()(x)
        return self._call_(arg)

    def domain(self):
        """
        Return the domain of this derivation.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: f = R.derivation(y); f
            d/dy
            sage: f.domain()
            Multivariate Polynomial Ring in x, y over Rational Field
            sage: f.domain() is R
            True
        """
        return self.parent().domain()

    def codomain(self):
        """
        Return the codomain of this derivation.

        EXAMPLES::

            sage: R.<x> = QQ[]
            sage: f = R.derivation(); f
            d/dx
            sage: f.codomain()
            Univariate Polynomial Ring in x over Rational Field
            sage: f.codomain() is R
            True

        ::

            sage: S.<y> = R[]
            sage: M = R.derivation_module(S)
            sage: M.random_element().codomain()
            Univariate Polynomial Ring in y over
             Univariate Polynomial Ring in x over Rational Field
            sage: M.random_element().codomain() is S
            True
        """
        return self.parent().codomain()


class RingDerivationWithoutTwist(RingDerivation):
    """
    An abstract class for untwisted derivations.
    """
    def _repr_(self):
        r"""
        Return a string representation of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: R.derivation(x)
            d/dx
            sage: R.derivation(y)
            d/dy
        """
        parent = self.parent()
        try:
            dual_basis = parent.dual_basis()
        except NotImplementedError:
            return "A derivation on %s" % parent.domain()
        coeffs = self.list()
        s = ""
        for i in range(len(dual_basis)):
            c = coeffs[i]
            sc = str(c)
            if sc == "0":
                continue
            name = str(dual_basis[i])
            if all(name.find(c) == -1 for c in "+-*/ ()"):
                ddx = "d/d%s" % name
            else:
                ddx = "d_{%s}" % name
            if sc == "1":
                s += " + " + ddx
            elif sc == "-1":
                s += " - " + ddx
            elif c._is_atomic() and sc[0] != "-":
                s += " + %s*%s" % (sc, ddx)
            elif (-c)._is_atomic():
                s += " - %s*%s" % (-c, ddx)
            else:
                s += " + (%s)*%s" % (sc, ddx)
        if s[:3] == " + ":
            return s[3:]
        if s[:3] == " - ":
            return "-" + s[3:]
        if s == "":
            return "0"
        return s

    def _latex_(self):
        r"""
        Return a LaTeX representation of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: ddx = R.derivation(x)
            sage: ddy = R.derivation(y)

            sage: latex(ddx)
            \frac{d}{dx}
            sage: latex(ddy)
            \frac{d}{dy}
            sage: latex(ddx + ddy)
            \frac{d}{dx} + \frac{d}{dy}
        """
        parent = self.parent()
        try:
            dual_basis = parent.dual_basis()
        except NotImplementedError:
            return "\\text{A derivation on } %s" % latex(parent.domain())
        coeffs = self.list()
        s = ""
        for i in range(len(dual_basis)):
            c = coeffs[i]
            sc = str(c)
            if sc == "0":
                continue
            name = str(dual_basis[i])
            if all(name.find(c) == -1 for c in "+-*/ ()"):
                ddx = "\\frac{d}{d%s}" % latex(dual_basis[i])
            else:
                ddx = "d_{%s}" % latex(dual_basis[i])
            if sc == "1":
                s += " + " + ddx
            elif sc == "-1":
                s += " - " + ddx
            elif c._is_atomic() and sc[0] != "-":
                s += " + %s %s" % (sc, ddx)
            elif (-c)._is_atomic():
                s += " - %s %s" % (-c, ddx)
            else:
                s += " + \\left(%s\\right) %s" % (sc, ddx)
        if s[:3] == " + ":
            return s[3:]
        if s[:3] == " - ":
            return "-" + s[3:]
        if s == "":
            return "0"
        return s

    def list(self):
        """
        Return the list of coefficient of this derivation
        on the canonical basis.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)

            sage: R.derivation(x).list()
            [1, 0]
            sage: R.derivation(y).list()
            [0, 1]

            sage: f = x*R.derivation(x) + y*R.derivation(y); f
            x*d/dx + y*d/dy
            sage: f.list()
            [x, y]
        """
        parent = self.parent()
        return [self(x) for x in parent.dual_basis()]

    def monomial_coefficients(self, copy=None):
        r"""
        Return dictionary of nonzero coordinates (on the canonical
        basis) of this derivation.

        More precisely, this returns a dictionary whose keys are indices
        of basis elements and whose values are the corresponding coefficients.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)

            sage: R.derivation(x).monomial_coefficients()
            {0: 1}
            sage: R.derivation(y).monomial_coefficients()
            {1: 1}

            sage: f = x*R.derivation(x) + y*R.derivation(y); f
            x*d/dx + y*d/dy
            sage: f.monomial_coefficients()
            {0: x, 1: y}
        """
        dual_basis = self.parent().dual_basis()
        dic = {}
        for i in range(len(dual_basis)):
            c = self(dual_basis[i])
            if c != 0:
                dic[i] = c
        return dic

    def is_zero(self) -> bool:
        """
        Return ``True`` if this derivation is zero.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(); f
            d/dx
            sage: f.is_zero()
            False

            sage: (f-f).is_zero()
            True
        """
        return all(c.is_zero() for c in self.list())

    def _richcmp_(self, other, op) -> bool:
        """
        Compare this derivation with ``other`` according
        to the comparison operator ``op``.

        EXAMPLES::

            sage: R.<x,y,z> = GF(5)[]
            sage: D = sum(v*R.derivation(v) for v in R.gens()); D
            x*d/dx + y*d/dy + z*d/dz
            sage: D.pth_power() == D
            True
        """
        if op == op_EQ:
            if isinstance(other, RingDerivationWithoutTwist):
                return self.list() == other.list()
            return False
        if op == op_NE:
            if isinstance(other, RingDerivationWithoutTwist):
                return self.list() != other.list()
            return True
        return NotImplemented

    def _bracket_(self, other):
        """
        Return the Lie bracket (that is the commutator) of
        this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx._bracket_(Dy)
            0

            sage: Dx.bracket(x*Dy)
            d/dy

        TESTS::

            sage: M = R.derivation_module()
            sage: X = M.random_element()
            sage: X.bracket(X)
            0

        We check the Jacobi identity::

            sage: Y = M.random_element()
            sage: Z = M.random_element()
            sage: X.bracket(Y.bracket(Z)) + Y.bracket(Z.bracket(X)) + Z.bracket(X.bracket(Y))
            0

        and the product rule::

            sage: f = R.random_element()
            sage: X.bracket(f*Y) == X(f)*Y + f*X.bracket(Y)
            True
        """
        parent = self.parent()
        if parent.domain() is not parent.codomain():
            raise TypeError("the bracket is only defined for derivations with same domain and codomain")
        arg = [self(other(x)) - other(self(x)) for x in parent.dual_basis()]
        return parent(arg)

    def pth_power(self):
        r"""
        Return the `p`-th power of this derivation where `p`
        is the characteristic of the domain.

        .. NOTE::

            Leibniz rule implies that this is again a derivation.

        EXAMPLES::

            sage: R.<x,y> = GF(5)[]
            sage: Dx = R.derivation(x)
            sage: Dx.pth_power()
            0
            sage: (x*Dx).pth_power()
            x*d/dx
            sage: (x^6*Dx).pth_power()
            x^26*d/dx

            sage: Dy = R.derivation(y)                                                  # needs sage.rings.finite_rings
            sage: (x*Dx + y*Dy).pth_power()                                             # needs sage.rings.finite_rings
            x*d/dx + y*d/dy

        An error is raised if the domain has characteristic zero::

            sage: R.<x,y> = QQ[]
            sage: Dx = R.derivation(x)
            sage: Dx.pth_power()
            Traceback (most recent call last):
            ...
            TypeError: the domain of the derivation must have positive and prime characteristic

        or if the characteristic is not a prime number::

            sage: R.<x,y> = Integers(10)[]
            sage: Dx = R.derivation(x)
            sage: Dx.pth_power()
            Traceback (most recent call last):
            ...
            TypeError: the domain of the derivation must have positive and prime characteristic

        TESTS::

            sage: R.<x,y> = GF(3)[]
            sage: D = R.derivation_module().random_element()
            sage: Dp = D.pth_power()
            sage: f = R.random_element()
            sage: Dp(f) == D(D(D(f)))
            True

            sage: D.bracket(Dp)                                                         # needs sage.rings.finite_rings
            0
        """
        parent = self.parent()
        if parent.domain() is not parent.codomain():
            raise TypeError("the derivation must have the same domain and codomain")
        p = parent.domain().characteristic()
        if not p.is_prime():
            raise TypeError("the domain of the derivation must have positive and prime characteristic")
        arg = [ ]
        for x in parent.dual_basis():
            res = x
            for _ in range(p):
                res = self(res)
            arg.append(res)
        return parent(arg)

    def precompose(self, morphism):
        r"""
        Return the derivation obtained by applying first
        ``morphism`` and then this derivation.

        INPUT:

        - ``morphism`` -- a homomorphism of rings whose codomain is
          the domain of this derivation or a ring that coerces to
          the domain of this derivation

        EXAMPLES::

            sage: A.<x> = QQ[]
            sage: B.<x,y> = QQ[]
            sage: D = B.derivation(x) - 2*x*B.derivation(y); D
            d/dx - 2*x*d/dy

        When restricting to ``A``, the term ``d/dy`` disappears
        (since it vanishes on ``A``)::

            sage: D.precompose(A)
            d/dx

        If we restrict to another well chosen subring, the derivation vanishes::

            sage: C.<t> = QQ[]
            sage: f = C.hom([x^2 + y]); f
            Ring morphism:
              From: Univariate Polynomial Ring in t over Rational Field
              To:   Multivariate Polynomial Ring in x, y over Rational Field
              Defn: t |--> x^2 + y
            sage: D.precompose(f)
            0

        Note that this method cannot be used to compose derivations::

            sage: D.precompose(D)
            Traceback (most recent call last):
            ...
            TypeError: you must give a homomorphism of rings

        TESTS::

            sage: D.precompose(C)
            Traceback (most recent call last):
            ...
            TypeError: the given ring does not coerce to the domain of the derivation
        """
        parent = self.parent()
        if morphism in Rings().Commutative():
            if parent.domain().has_coerce_map_from(morphism):
                morphism = parent.domain().coerce_map_from(morphism)
            else:
                raise TypeError("the given ring does not coerce to the domain of the derivation")
        elif not (isinstance(morphism, Map) and morphism.category_for().is_subcategory(Rings())):
            raise TypeError("you must give a homomorphism of rings")
        M = RingDerivationModule(morphism.domain(), parent.defining_morphism() * morphism)
        arg = [self(morphism(x)) for x in M.dual_basis()]
        return M(arg)

    def postcompose(self, morphism):
        """
        Return the derivation obtained by applying first
        this derivation and then ``morphism``.

        INPUT:

        - ``morphism`` -- a homomorphism of rings whose domain is
          the codomain of this derivation or a ring into which the
          codomain of this derivation coerces

        EXAMPLES::

            sage: A.<x,y>= QQ[]
            sage: ev = A.hom([QQ(0), QQ(1)])
            sage: Dx = A.derivation(x)
            sage: Dy = A.derivation(y)

        We can define the derivation at `(0,1)` just by postcomposing
        with ``ev``::

            sage: dx = Dx.postcompose(ev)
            sage: dy = Dy.postcompose(ev)
            sage: f = x^2 + y^2
            sage: dx(f)
            0
            sage: dy(f)
            2

        Note that we cannot avoid the creation of the evaluation morphism:
        if we pass in ``QQ`` instead, an error is raised since there is
        no coercion morphism from ``A`` to ``QQ``::

            sage: Dx.postcompose(QQ)
            Traceback (most recent call last):
            ...
            TypeError: the codomain of the derivation does not coerce to the given ring

        Note that this method cannot be used to compose derivations::

            sage: Dx.precompose(Dy)
            Traceback (most recent call last):
            ...
            TypeError: you must give a homomorphism of rings
        """
        parent = self.parent()
        if morphism in Rings().Commutative():
            if morphism.has_coerce_map_from(parent.codomain()):
                morphism = morphism.coerce_map_from(parent.codomain())
            else:
                raise TypeError("the codomain of the derivation does not coerce to the given ring")
        elif not (isinstance(morphism, Map) and morphism.category_for().is_subcategory(Rings())):
            raise TypeError("you must give a homomorphism of rings")
        M = RingDerivationModule(parent.domain(), morphism * parent.defining_morphism())
        arg = [morphism(self(x)) for x in M.dual_basis()]
        return M(arg)

    def extend_to_fraction_field(self):
        r"""
        Return the extension of this derivation to fraction fields of
        the domain and the codomain.

        EXAMPLES::

            sage: S.<x> = QQ[]
            sage: d = S.derivation()
            sage: d
            d/dx

            sage: D = d.extend_to_fraction_field()
            sage: D
            d/dx
            sage: D.domain()
            Fraction Field of Univariate Polynomial Ring in x over Rational Field

            sage: D(1/x)
            -1/x^2
        """
        parent = self.parent()
        domain = parent.domain().fraction_field()
        codomain = parent.codomain().fraction_field()
        M = RingDerivationModule(domain, codomain)
        try:
            return M(self)
        except (ValueError, NotImplementedError):
            return M(self.list())


class RingDerivationWithoutTwist_zero(RingDerivationWithoutTwist):
    """
    This class can only represent the zero derivation.

    It is used when the parent is the zero derivation module
    (e.g., when its domain is ``ZZ``, ``QQ``, a finite field, etc.)
    """
    def __init__(self, parent, arg=None):
        """
        Initialize this derivation.

        TESTS::

            sage: M = ZZ.derivation_module()
            sage: der = M(); der
            0

            sage: from sage.rings.derivation import RingDerivationWithoutTwist_zero
            sage: isinstance(der, RingDerivationWithoutTwist_zero)
            True

            sage: TestSuite(der).run()
        """
        if isinstance(arg, list) and len(arg) == 1 and isinstance(arg[0], RingDerivation):
            arg = arg[0]
        if arg and not (isinstance(arg, RingDerivation) and arg.is_zero()):
            raise ValueError("unable to create the derivation")
        RingDerivation.__init__(self, parent)

    def _repr_(self):
        """
        Return a string representation of this derivation.

        EXAMPLES::

            sage: M = ZZ.derivation_module()
            sage: M()
            0
        """
        return "0"

    def _latex_(self):
        """
        Return a string representation of this derivation.

        EXAMPLES::

            sage: M = ZZ.derivation_module()
            sage: latex(M())
            0
        """
        return "0"

    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(x)
            sage: hash(f)  # random
            3713081631936575706
        """
        return hash(tuple(self.list()))

    def _add_(self, other):
        """
        Return the sum of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx + Dy
            d/dx + d/dy
        """
        return other

    def _sub_(self, other):
        """
        Return the difference of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx - Dy
            d/dx - d/dy
        """
        return -other

    def _neg_(self):
        """
        Return the opposite of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: -Dx
            -d/dx
        """
        return self

    def _lmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dx * 2
            2*d/dx
            sage: Dx * x^2
            x^2*d/dx
        """
        return self

    def _rmul_(self, left):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: 2 * Dx
            2*d/dx
            sage: x^2 * Dx
            x^2*d/dx
        """
        return self

    def _call_(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = x*R.derivation(x) + y*R.derivation(y)
            sage: f(x^2 + 3*x*y - y^2)
            2*x^2 + 6*x*y - 2*y^2
        """
        return self.parent().codomain().zero()

    def _bracket_(self, other):
        """
        Return the Lie bracket (that is the commutator) of
        this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx._bracket_(Dy)
            0
        """
        return self

    def is_zero(self) -> bool:
        """
        Return ``True`` if this derivation vanishes.

        EXAMPLES::

            sage: M = QQ.derivation_module()
            sage: M().is_zero()
            True
        """
        return True

    def list(self):
        """
        Return the list of coefficient of this derivation
        on the canonical basis.

        EXAMPLES::

            sage: M = QQ.derivation_module()
            sage: M().list()
            []
        """
        return []


class RingDerivationWithoutTwist_wrapper(RingDerivationWithoutTwist):
    """
    This class is a wrapper for derivation.

    It is useful for changing the parent without changing the
    computation rules for derivations. It is used for derivations
    over fraction fields and quotient rings.
    """
    def __init__(self, parent, arg=None):
        """
        Initialize this derivation.

        TESTS::

            sage: from sage.rings.derivation import RingDerivationWithoutTwist_wrapper
            sage: R.<x,y> = GF(5)[]
            sage: S = R.quo([x^5, y^5])
            sage: M = S.derivation_module()
            sage: der = M.random_element()
            sage: isinstance(der, RingDerivationWithoutTwist_wrapper)
            True
            sage: TestSuite(der).run()
        """
        if isinstance(arg, list) and len(arg) == 1 and isinstance(arg[0], RingDerivation):
            arg = arg[0]
        if isinstance(arg, RingDerivationWithoutTwist_wrapper):
            self._base_derivation = arg._base_derivation
        else:
            self._base_derivation = parent._base_derivation(arg)
        RingDerivation.__init__(self, parent)

    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(x)
            sage: hash(f)  # random
            3713081631936575706
        """
        return hash(tuple(self.list()))

    def _add_(self, other):
        """
        Return the sum of this derivation and ``other``.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: Dx = S.derivation(x)
            sage: Dy = S.derivation(y)
            sage: Dx + Dy
            d/dx + d/dy
        """
        return type(self)(self.parent(), self._base_derivation + other._base_derivation)

    def _sub_(self, other):
        """
        Return the difference of this derivation and ``other``.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: Dx = S.derivation(x)
            sage: Dy = S.derivation(y)
            sage: Dx - Dy
            d/dx - d/dy
        """
        return type(self)(self.parent(), self._base_derivation - other._base_derivation)

    def _neg_(self):
        """
        Return the opposite of this derivation.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: Dx = S.derivation(x)
            sage: -Dx
            -d/dx
        """
        return type(self)(self.parent(), -self._base_derivation)

    def _lmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: Dx = S.derivation(x)
            sage: Dx * 2
            2*d/dx
            sage: Dx * x^2
            x^2*d/dx
        """
        return type(self)(self.parent(), self._base_derivation * factor)

    def _rmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: Dx = S.derivation(x)
            sage: 2 * Dx
            2*d/dx
            sage: x^2 * Dx
            x^2*d/dx
        """
        return type(self)(self.parent(), factor * self._base_derivation)

    def list(self):
        """
        Return the list of coefficient of this derivation
        on the canonical basis.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: M = S.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)
            sage: S.derivation(x).list()
            [1, 0]
            sage: S.derivation(y).list()
            [0, 1]
            sage: f = x*S.derivation(x) + y*S.derivation(y); f
            x*d/dx + y*d/dy
            sage: f.list()
            [x, y]
        """
        return self._base_derivation.list()


class RingDerivationWithoutTwist_function(RingDerivationWithoutTwist):
    """
    A class for untwisted derivations over rings whose elements
    are either polynomials, rational fractions, power series or
    Laurent series.
    """
    def __init__(self, parent, arg=None):
        """
        Initialize this derivation.

        TESTS::

            sage: R.<x,y> = ZZ[]
            sage: R.derivation(x)
            d/dx
            sage: der = R.derivation([1,2])
            sage: der
            d/dx + 2*d/dy

            sage: TestSuite(der).run()
        """
        domain = parent.domain()
        codomain = parent.codomain()
        ngens = domain.ngens()
        self._base_derivation = parent._base_derivation()
        self._images = [codomain.zero() for _ in range(ngens)]
        if arg is None:
            arg = domain.gen()
        if isinstance(arg, list) and len(arg) == 1 and isinstance(arg[0], RingDerivation):
            arg = arg[0]
        if not arg:
            pass
        elif (isinstance(arg, RingDerivationWithoutTwist_function)
              and parent.has_coerce_map_from(arg.parent())):
            self._base_derivation = parent._base_derivation(arg._base_derivation)
            self._images = [codomain(x) for x in arg._images]
        elif isinstance(arg, (tuple, list)):
            if len(arg) < ngens:
                raise ValueError("the number of images is incorrect")
            self._base_derivation = parent._base_derivation(arg[:-ngens])
            self._images = [codomain(x) for x in arg[-ngens:]]
        else:
            for i in range(ngens):
                if arg == domain.gen(i):
                    self._base_derivation = parent._base_derivation()
                    self._images[i] = codomain.one()
                    break
            else:
                self._base_derivation = parent._base_derivation(arg)
        RingDerivation.__init__(self, parent)

    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(x)
            sage: hash(f)  # random
            3713081631936575706
        """
        return hash(tuple(self.list()))

    def _add_(self, other):
        """
        Return the sum of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx + Dy
            d/dx + d/dy
        """
        base_derivation = self._base_derivation + other._base_derivation
        im = [ self._images[i] + other._images[i] for i in range(self.parent().domain().ngens()) ]
        return type(self)(self.parent(), [base_derivation] + im)

    def _sub_(self, other):
        """
        Return the subtraction of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dy = R.derivation(y)
            sage: Dx - Dy
            d/dx - d/dy
        """
        base_derivation = self._base_derivation - other._base_derivation
        im = [ self._images[i] - other._images[i] for i in range(self.parent().domain().ngens()) ]
        return type(self)(self.parent(), [base_derivation] + im)

    def _rmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: 2 * Dx
            2*d/dx
            sage: x^2 * Dx
            x^2*d/dx
        """
        factor = self.parent().codomain()(factor)
        base_derivation = factor * self._base_derivation
        im = [ factor*x for x in self._images ]
        return type(self)(self.parent(), [base_derivation] + im)

    def _lmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: Dx = R.derivation(x)
            sage: Dx * 2
            2*d/dx
            sage: Dx * x^2
            x^2*d/dx
        """
        return self._rmul_(factor)

    def _call_(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: D = x*R.derivation(x) + y*R.derivation(y)
            sage: D(x^2 + 3*x*y - y^2)
            2*x^2 + 6*x*y - 2*y^2
        """
        parent = self.parent()
        domain = parent.domain()
        codomain = parent.codomain()
        defining_morphism = parent.defining_morphism()
        if isinstance(domain, FractionField_generic):
            num = x.numerator()
            den = x.denominator()
            u = defining_morphism(num)
            v = defining_morphism(den)
            up = num.map_coefficients(self._base_derivation, codomain)(*domain.gens())
            vp = den.map_coefficients(self._base_derivation, codomain)(*domain.gens())
            res = (up*v - u*vp) / (v*v)
        else:
            res = x.map_coefficients(self._base_derivation, codomain)(*domain.gens())
        for i in range(len(self._images)):
            res += defining_morphism(x.derivative(domain.gen(i))) * self._images[i]
        return res

    def is_zero(self) -> bool:
        """
        Return ``True`` if this derivation is zero.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(); f
            d/dx
            sage: f.is_zero()
            False

            sage: (f-f).is_zero()
            True
        """
        if not self._base_derivation.is_zero():
            return False
        return all(im == 0 for im in self._images)

    def list(self):
        """
        Return the list of coefficient of this derivation
        on the canonical basis.

        EXAMPLES::

            sage: R.<x,y> = GF(5)[[]]
            sage: M = R.derivation_module()
            sage: M.basis()
            Family (d/dx, d/dy)

            sage: R.derivation(x).list()
            [1, 0]
            sage: R.derivation(y).list()
            [0, 1]

            sage: f = x*R.derivation(x) + y*R.derivation(y); f
            x*d/dx + y*d/dy
            sage: f.list()
            [x, y]
        """
        return self._base_derivation.list() + self._images


class RingDerivationWithoutTwist_fraction_field(RingDerivationWithoutTwist_wrapper):
    """
    This class handles derivations over fraction fields.
    """
    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(x)
            sage: hash(f)  # random
            3713081631936575706
        """
        return hash(tuple(self.list()))

    def _call_(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: K = R.fraction_field()
            sage: f = K.derivation(); f
            d/dx
            sage: f(1/x)
            (-1)/x^2
        """
        defining_morphism = self.parent().defining_morphism()
        num = x.numerator()
        den = x.denominator()
        u = defining_morphism(num)
        v = defining_morphism(den)
        up = self._base_derivation(u)
        vp = self._base_derivation(v)
        return (up*v - u*vp) / (v*v)


class RingDerivationWithoutTwist_quotient(RingDerivationWithoutTwist_wrapper):
    """
    This class handles derivations over quotient rings.
    """
    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: f = R.derivation(x)
            sage: hash(f)  # random
            3713081631936575706
        """
        return hash(tuple(self.list()))

    def _call_(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<X,Y> = GF(5)[]
            sage: S.<x,y> = R.quo([X^5, Y^5])
            sage: f = x^3*S.derivation(); f
            x^3*d/dx
            sage: f(x^3)
            0
        """
        return self._base_derivation(x.lift())


class RingDerivationWithTwist_generic(RingDerivation):
    r"""
    The class handles `\theta`-derivations of the form
    `\lambda (\theta - \iota)` (where `\iota` is the defining
    morphism of the codomain over the domain) for a scalar
    `\lambda` varying in the codomain.
    """
    def __init__(self, parent, scalar=0):
        """
        Initialize this derivation.

        TESTS::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: R.derivation(twist=theta)
            0
            sage: R.derivation(1, twist=theta)
            [x |--> y, y |--> x] - id

            sage: der = R.derivation(x, twist=theta)
            sage: TestSuite(der).run()
        """
        codomain = parent.codomain()
        self._scalar = codomain(scalar)
        RingDerivation.__init__(self, parent)

    def __hash__(self):
        """
        Return a hash of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: f = R.derivation(1, twist=theta)
            sage: hash(f)  # random
            -6511057926760520014
        """
        return hash(self._scalar)

    def _repr_(self):
        r"""
        Return a string representation of this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: R.derivation(1, twist=theta)
            [x |--> y, y |--> x] - id
        """
        scalar = self._scalar
        sc = str(scalar)
        if sc == "0":
            return "0"
        defining_morphism = self.parent().defining_morphism()
        twisting_morphism = self.parent().twisting_morphism()
        try:
            if defining_morphism.is_identity():
                sdef = "id"
            else:
                sdef = "[%s]" % defining_morphism._repr_short()
        except (AttributeError, NotImplementedError):
            sdef = "defining_morphism"
        try:
            stwi = "[%s]" % twisting_morphism._repr_short()
        except AttributeError:
            stwi = "twisting_morphism"
        if sc == "1":
            return "%s - %s" % (stwi, sdef)
        if sc == "-1":
            s = "-"
        elif scalar._is_atomic():
            s = "%s*" % sc
        elif (-scalar)._is_atomic():
            s = "-%s*" % (-scalar)
        else:
            s = "(%s)*" % sc
        return "%s(%s - %s)" % (s, stwi, sdef)

    def _latex_(self):
        r"""
        Return a LaTeX representation of this derivation.

        EXAMPLES::

            sage: k.<a> = GF(5^3)
            sage: Frob = k.frobenius_endomorphism()
            sage: der = k.derivation(a + 1, twist=Frob)
            sage: latex(der)
            \left(a + 1\right)  \left(\left[a \mapsto a^{5}\right] - \text{id}\right)
        """
        scalar = self._scalar
        sc = str(scalar)
        if sc == "0":
            return "0"
        defining_morphism = self.parent().defining_morphism()
        twisting_morphism = self.parent().twisting_morphism()
        try:
            if defining_morphism.is_identity():
                sdef = "\\text{id}"
            else:
                sdef = "\\left[%s\\right]" % latex(defining_morphism)
        except (AttributeError, NotImplementedError):
            sdef = "\\text{defining morphism}"
        try:
            stwi = "\\left[%s\\right]" % latex(twisting_morphism)
        except AttributeError:
            stwi = "\\text{twisting morphism}"
        if sc == "1":
            return "%s - %s" % (stwi, sdef)
        if sc == "-1":
            s = "-"
        elif scalar._is_atomic():
            s = "%s " % sc
        elif (-scalar)._is_atomic():
            s = "-%s " % (-scalar)
        else:
            s = "\\left(%s\\right) " % sc
        return "%s \\left(%s - %s\\right)" % (s, stwi, sdef)

    def _add_(self, other):
        """
        Return the sum of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: der1 = R.derivation(x, twist=theta); der1
            x*([x |--> y, y |--> x] - id)
            sage: der2 = R.derivation(y, twist=theta); der2
            y*([x |--> y, y |--> x] - id)
            sage: der1 + der2
            (x + y)*([x |--> y, y |--> x] - id)
        """
        return type(self)(self.parent(), self._scalar + other._scalar)

    def _sub_(self, other):
        """
        Return the subtraction of this derivation and ``other``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: der1 = R.derivation(x, twist=theta); der1
            x*([x |--> y, y |--> x] - id)
            sage: der2 = R.derivation(y, twist=theta); der2
            y*([x |--> y, y |--> x] - id)
            sage: der1 - der2
            (x - y)*([x |--> y, y |--> x] - id)

        TESTS::

            sage: der1 - der1
            0
            sage: der2 - der2
            0
        """
        return type(self)(self.parent(), self._scalar - other._scalar)

    def _rmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: D = R.derivation(x, twist=theta); D
            x*([x |--> y, y |--> x] - id)
            sage: y * D
            x*y*([x |--> y, y |--> x] - id)
        """
        return type(self)(self.parent(), factor * self._scalar)

    def _lmul_(self, factor):
        """
        Return the product of this derivation by the scalar ``factor``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: D = R.derivation(x, twist=theta); D
            x*([x |--> y, y |--> x] - id)
            sage: D * y
            x*y*([x |--> y, y |--> x] - id)
        """
        return self._rmul_(factor)

    def _call_(self, x):
        """
        Return the image of ``x`` under this derivation.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: f = R.derivation(1, twist=theta); f
            [x |--> y, y |--> x] - id
            sage: f(x)
            -x + y
        """
        parent = self.parent()
        return self._scalar * (parent.twisting_morphism()(x) - parent.defining_morphism()(x))

    def list(self):
        """
        Return the list of coefficient of this twisted derivation
        on the canonical basis.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: K = R.fraction_field()
            sage: theta = K.hom([y,x])
            sage: M = K.derivation_module(twist=theta)
            sage: M.basis()
            Family (twisting_morphism - id,)
            sage: f = (x+y) * M.gen()
            sage: f
            (x + y)*(twisting_morphism - id)
            sage: f.list()
            [x + y]
        """
        return [ self._scalar ]

    def precompose(self, morphism):
        r"""
        Return the twisted derivation obtained by applying first
        ``morphism`` and then this twisted derivation.

        INPUT:

        - ``morphism`` -- a homomorphism of rings whose codomain is
          the domain of this derivation or a ring that coerces to
          the domain of this derivation

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: D = R.derivation(x, twist=theta); D
            x*([x |--> y, y |--> x] - id)

            sage: f = R.hom([x^2, y^3])
            sage: g = D.postcompose(f); g
            x^2*([x |--> y^3, y |--> x^2] - [x |--> x^2, y |--> y^3])

        Observe that the `g` is no longer a `\theta`-derivation but
        a `(f \circ \theta)`-derivation::

            sage: g.parent().twisting_morphism()
            Ring endomorphism of Multivariate Polynomial Ring in x, y over Integer Ring
              Defn: x |--> y^3
                    y |--> x^2
        """
        parent = self.parent()
        if morphism in Rings().Commutative():
            if parent.domain().has_coerce_map_from(morphism):
                morphism = parent.domain().coerce_map_from(morphism)
            else:
                raise TypeError("the given ring does not coerce to the domain of the derivation")
        elif not (isinstance(morphism, Map) and morphism.category_for().is_subcategory(Rings())):
            raise TypeError("you must give a homomorphism of rings")
        M = RingDerivationModule(morphism.domain(), parent.defining_morphism() * morphism,
                                 parent.twisting_morphism() * morphism)
        return M(self._scalar)

    def postcompose(self, morphism):
        r"""
        Return the twisted derivation obtained by applying first
        this twisted derivation and then ``morphism``.

        INPUT:

        - ``morphism`` -- a homomorphism of rings whose domain is
          the codomain of this derivation or a ring into which the
          codomain of this derivation

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: D = R.derivation(x, twist=theta); D
            x*([x |--> y, y |--> x] - id)

            sage: f = R.hom([x^2, y^3])
            sage: g = D.precompose(f); g
            x*([x |--> y^2, y |--> x^3] - [x |--> x^2, y |--> y^3])

        Observe that the `g` is no longer a `\theta`-derivation but
        a `(\theta \circ f)`-derivation::

            sage: g.parent().twisting_morphism()
            Ring endomorphism of Multivariate Polynomial Ring in x, y over Integer Ring
              Defn: x |--> y^2
                    y |--> x^3
        """
        parent = self.parent()
        if morphism in Rings().Commutative():
            if morphism.has_coerce_map_from(parent.codomain()):
                morphism = morphism.coerce_map_from(parent.codomain())
            else:
                raise TypeError("the codomain of the derivation does not coerce to the given ring")
        elif not (isinstance(morphism, Map) and morphism.category_for().is_subcategory(Rings())):
            raise TypeError("you must give a homomorphism of rings")
        M = RingDerivationModule(parent.domain(), morphism * parent.defining_morphism(),
                                 morphism * parent.twisting_morphism())
        return M(morphism(self._scalar))

    def _richcmp_(self, other, op):
        """
        Compare this derivation with ``other`` according
        to the comparison operator ``op``.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: Dx = R.derivation(x, twist=theta); Dx
            x*([x |--> y, y |--> x] - id)
            sage: Dy = R.derivation(y, twist=theta); Dy
            y*([x |--> y, y |--> x] - id)
            sage: D = R.derivation(x+y, twist=theta); D
            (x + y)*([x |--> y, y |--> x] - id)

            sage: Dx == Dy
            False
            sage: D == Dx + Dy
            True

            sage: D != Dy
            True
        """
        if op == op_EQ:
            if isinstance(other, RingDerivationWithTwist_generic):
                return self._scalar == other._scalar
            return False
        if op == op_NE:
            if isinstance(other, RingDerivationWithTwist_generic):
                return self._scalar != other._scalar
            return True
        return NotImplemented

    def extend_to_fraction_field(self):
        r"""
        Return the extension of this derivation to fraction fields of
        the domain and the codomain.

        EXAMPLES::

            sage: R.<x,y> = ZZ[]
            sage: theta = R.hom([y,x])
            sage: d = R.derivation(x, twist=theta)
            sage: d
            x*([x |--> y, y |--> x] - id)

            sage: D = d.extend_to_fraction_field(); D                                   # needs sage.libs.singular
            x*([x |--> y, y |--> x] - id)
            sage: D.domain()                                                            # needs sage.libs.singular
            Fraction Field of Multivariate Polynomial Ring in x, y over Integer Ring

            sage: D(1/x)                                                                # needs sage.libs.singular
            (x - y)/y
        """
        parent = self.parent()
        domain = parent.domain().fraction_field()
        codomain = parent.codomain().fraction_field()
        twist = parent.twisting_morphism().extend_to_fraction_field()
        M = RingDerivationModule(domain, codomain, twist)
        return M(codomain(self._scalar))



"""
List plots
"""

from sage.structure.element import Matrix
from sage.matrix.constructor import matrix
from sage.rings.real_double import RDF


def list_plot3d(v, interpolation_type='default', point_list=None, **kwds):
    r"""
    A 3-dimensional plot of a surface defined by the list `v`
    of points in 3-dimensional space.

    INPUT:

    - ``v`` -- something that defines a set of points in 3 space:

      - a matrix

      - a list of 3-tuples

      - a list of lists (all of the same length) -- this is treated the same as
        a matrix

    OPTIONAL KEYWORDS:

    - ``interpolation_type`` -- 'linear', 'clough' (CloughTocher2D), 'spline'

      'linear' will perform linear interpolation

      The option 'clough' will interpolate by using a piecewise cubic
      interpolating Bezier polynomial on each triangle, using a
      Clough-Tocher scheme.  The interpolant is guaranteed to be
      continuously differentiable.  The gradients of the interpolant
      are chosen so that the curvature of the interpolating surface is
      approximately minimized.

      The option 'spline' interpolates using a bivariate B-spline.

      When v is a matrix the default is to use linear interpolation, when
      v is a list of points the default is 'clough'.

    - ``degree`` -- integer between 1 and 5, controls the degree of spline
      used for spline interpolation. For data that is highly oscillatory
      use higher values

    - ``point_list`` -- if ``point_list=True`` is passed, then if the array
      is a list of lists of length three, it will be treated as an
      array of points rather than a 3xn array.

    - ``num_points`` -- number of points to sample interpolating
      function in each direction, when ``interpolation_type`` is not
      ``default``. By default for an `n\times n` array this is `n`.

    - ``**kwds`` -- all other arguments are passed to the surface function

    OUTPUT: a 3d plot

    EXAMPLES:

    We plot a matrix that illustrates summation modulo `n`::

        sage: n = 5
        sage: list_plot3d(matrix(RDF, n, [(i+j)%n for i in [1..n] for j in [1..n]]))
        Graphics3d Object

    .. PLOT::

        sphinx_plot(list_plot3d(matrix(RDF, 5, [(i+j)%5 for i in range(1,6) for j in range(1,6)])))

    We plot a matrix of values of ``sin``::

        sage: from math import pi
        sage: m = matrix(RDF, 6, [sin(i^2 + j^2)
        ....:                     for i in [0,pi/5,..,pi] for j in [0,pi/5,..,pi]])
        sage: list_plot3d(m, color='yellow', frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        pi = float(pi)
        import numpy as np
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', frame_aspect_ratio=[1, 1, 1/3]))

    Though it does not change the shape of the graph, increasing
    ``num_points`` can increase the clarity of the graph::

        sage: list_plot3d(m, color='yellow', num_points=40,
        ....:             frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        pi = float(pi)
        import numpy as np
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', frame_aspect_ratio=[1, 1, 1/3], num_points=40))

    We can change the interpolation type::

        sage: import warnings
        sage: warnings.simplefilter('ignore', UserWarning)
        sage: list_plot3d(m, color='yellow', interpolation_type='clough',
        ....:             frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        import warnings
        warnings.simplefilter('ignore', UserWarning)
        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='clough', frame_aspect_ratio=[1, 1, 1/3]))

    We can make this look better by increasing the number of samples::

        sage: list_plot3d(m, color='yellow', interpolation_type='clough',
        ....:             frame_aspect_ratio=[1, 1, 1/3], num_points=40)
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='clough', frame_aspect_ratio=[1, 1, 1/3], num_points=40))

    Let us try a spline::

        sage: list_plot3d(m, color='yellow', interpolation_type='spline',
        ....:             frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='spline', frame_aspect_ratio=[1, 1, 1/3]))

    That spline does not capture the oscillation very well; let's try a
    higher degree spline::

        sage: list_plot3d(m, color='yellow', interpolation_type='spline', degree=5,
        ....:             frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='spline', degree=5, frame_aspect_ratio=[1, 1, 1/3]))

    We plot a list of lists::

        sage: show(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]]))

    .. PLOT::

        sphinx_plot(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]]))

    We plot a list of points.  As a first example we can extract the
    (x,y,z) coordinates from the above example and make a list plot
    out of it. By default we do linear interpolation::

        sage: l = []
        sage: for i in range(6):
        ....:     for j in range(6):
        ....:         l.append((float(i*pi/5), float(j*pi/5), m[i, j]))
        sage: list_plot3d(l, color='red')
        Graphics3d Object

    .. PLOT::

        l = []
        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        for i in range(6):
             for j in range(6):
                 l.append((float(i*pi/5), float(j*pi/5), m[i, j]))
        sphinx_plot(list_plot3d(l, color='red'))

    Note that the points do not have to be regularly sampled. For example::

        sage: l = []
        sage: for i in range(-5, 5):
        ....:     for j in range(-5, 5):
        ....:         l.append((normalvariate(0, 1),
        ....:                   normalvariate(0, 1),
        ....:                   normalvariate(0, 1)))
        sage: L = list_plot3d(l, interpolation_type='clough',
        ....:                 color='orange', num_points=100)
        sage: L
        Graphics3d Object

    .. PLOT::

        l = []
        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        for i in range(-5, 5):
            for j in range(-5, 5):
                l.append((normalvariate(0, 1), normalvariate(0, 1), normalvariate(0, 1)))
        L = list_plot3d(l, interpolation_type='clough', color='orange', num_points=100)
        sphinx_plot(L)


    Check that no NaNs are produced (see :issue:`13135`)::

        sage: any(math.isnan(c) for v in L.vertices() for c in v)
        False

    TESTS:

    We plot 0, 1, and 2 points::

        sage: list_plot3d([])
        Graphics3d Object

        sage: list_plot3d([(2, 3, 4)])
        Graphics3d Object

        sage: list_plot3d([(0, 0, 1), (2, 3, 4)])
        Graphics3d Object

    However, if two points are given with the same x,y coordinates but
    different z coordinates, an exception will be raised::

        sage: pts = [(-4/5, -2/5, -2/5), (-4/5, -2/5, 2/5), (-4/5, 2/5, -2/5), (-4/5, 2/5, 2/5), (-2/5, -4/5, -2/5), (-2/5, -4/5, 2/5), (-2/5, -2/5, -4/5), (-2/5, -2/5, 4/5), (-2/5, 2/5, -4/5), (-2/5, 2/5, 4/5), (-2/5, 4/5, -2/5), (-2/5, 4/5, 2/5), (2/5, -4/5, -2/5), (2/5, -4/5, 2/5), (2/5, -2/5, -4/5), (2/5, -2/5, 4/5), (2/5, 2/5, -4/5), (2/5, 2/5, 4/5), (2/5, 4/5, -2/5), (2/5, 4/5, 2/5), (4/5, -2/5, -2/5), (4/5, -2/5, 2/5), (4/5, 2/5, -2/5), (4/5, 2/5, 2/5)]
        sage: show(list_plot3d(pts, interpolation_type='clough'))
        Traceback (most recent call last):
        ...
        ValueError: points with same x,y coordinates and different
        z coordinates were given. Interpolation cannot handle this.

    Additionally we need at least 3 points to do the interpolation::

        sage: mat = matrix(RDF, 1, 2, [3.2, 1.550])
        sage: show(list_plot3d(mat, interpolation_type='clough'))
        Traceback (most recent call last):
        ...
        ValueError: we need at least 3 points to perform the interpolation
    """
    import numpy
    if isinstance(v, Matrix):
        if (interpolation_type == 'default' or
                interpolation_type == 'linear' and 'num_points' not in kwds):
            return list_plot3d_matrix(v, **kwds)
        data = [(i, j, v[i, j])
                for i in range(v.nrows())
                for j in range(v.ncols())]
        return list_plot3d_tuples(data, interpolation_type, **kwds)

    if isinstance(v, numpy.ndarray):
        return list_plot3d(matrix(v), interpolation_type, **kwds)

    if isinstance(v, list):
        if not v:
            # return empty 3d graphic
            from .base import Graphics3d
            return Graphics3d()
        if len(v) == 1:
            # return a point
            from .shapes2 import point3d
            return point3d(v[0], **kwds)
        if len(v) == 2:
            # return a line
            from .shapes2 import line3d
            return line3d(v, **kwds)
        if isinstance(v[0], tuple) or point_list and len(v[0]) == 3:
            return list_plot3d_tuples(v, interpolation_type, **kwds)
        return list_plot3d_array_of_arrays(v, interpolation_type, **kwds)
    raise TypeError("v must be a matrix or list")


def list_plot3d_matrix(m, **kwds):
    """
    A 3-dimensional plot of a surface defined by a matrix ``M``
    defining points in 3-dimensional space.

    See :func:`list_plot3d` for full details.

    INPUT:

    - ``M`` -- a matrix

    OPTIONAL KEYWORDS:

    - ``**kwds`` -- all other arguments are passed to the surface function

    OUTPUT: a 3d plot

    EXAMPLES:

    We plot a matrix that illustrates summation modulo `n`::

        sage: n = 5
        sage: list_plot3d(matrix(RDF, n, [(i+j) % n         # indirect doctest
        ....:                             for i in [1..n] for j in [1..n]]))
        Graphics3d Object

    .. PLOT::

        sphinx_plot(list_plot3d(matrix(RDF, 5, [(i+j)%5 for i in range(1,6) for j in range(1,6)])))

    The interpolation type for matrices is 'linear'; for other types
    use other :func:`list_plot3d` input types.

    We plot a matrix of values of `sin`::

        sage: from math import pi
        sage: m = matrix(RDF, 6, [sin(i^2 + j^2)
        ....:                     for i in [0,pi/5,..,pi] for j in [0,pi/5,..,pi]])
        sage: list_plot3d(m, color='yellow', frame_aspect_ratio=[1, 1, 1/3])  # indirect doctest
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', frame_aspect_ratio=[1, 1, 1/3]))

    ::
        sage: list_plot3d(m, color='yellow', interpolation_type='linear')  # indirect doctest
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='linear'))

    Here is a colored example, using a colormap and a coloring function
    which must take values in (0, 1)::

        sage: cm = colormaps.rainbow
        sage: n = 20
        sage: cf = lambda x, y: ((2*(x-y)/n)**2) % 1
        sage: list_plot3d(matrix(RDF, n, [cos(pi*(i+j)/n) for i in [1..n]
        ....:                             for j in [1..n]]), color=(cf,cm))
        Graphics3d Object

    .. PLOT::

        cm = colormaps.rainbow
        cf = lambda x, y: ((2*(x-y)/20)**2) % 1
        expl = list_plot3d(matrix(RDF,20,20,[cos(pi*(i+j)/20) for i in range(1,21) for j in range(1,21)]),color=(cf,cm))
        sphinx_plot(expl)
    """
    from .parametric_surface import ParametricSurface

    def f(i, j):
        return (i, j, float(m[int(i), int(j)]))
    G = ParametricSurface(f, (list(range(m.nrows())), list(range(m.ncols()))),
                          **kwds)
    G._set_extra_kwds(kwds)
    return G


def list_plot3d_array_of_arrays(v, interpolation_type, **kwds):
    """
    A 3-dimensional plot of a surface defined by a list of lists ``v``
    defining points in 3-dimensional space.

    This is done by making the list of lists into a matrix and passing
    back to :func:`list_plot3d`.  See :func:`list_plot3d` for full
    details.

    INPUT:

    - ``v`` -- list of lists, all the same length
    - ``interpolation_type`` -- (default: ``'linear'``)

    OPTIONAL KEYWORDS:

    - ``**kwds`` -- all other arguments are passed to the surface function

    OUTPUT: a 3d plot

    EXAMPLES:

    The resulting matrix does not have to be square::

        sage: show(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1]])) # indirect doctest

    .. PLOT::

        sphinx_plot(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1]]))

    The normal route is for the list of lists to be turned into a matrix
    and use :func:`list_plot3d_matrix`::

        sage: show(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]]))

    .. PLOT::

        sphinx_plot(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]]))

    With certain extra keywords (see :func:`list_plot3d_matrix`), this function
    will end up using :func:`list_plot3d_tuples`::

        sage: show(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]],
        ....:                  interpolation_type='spline'))

    .. PLOT::

        sphinx_plot(list_plot3d([[1, 1, 1, 1], [1, 2, 1, 2], [1, 1, 3, 1], [1, 2, 1, 4]], interpolation_type='spline'))
    """
    m = matrix(RDF, len(v), len(v[0]), v)
    G = list_plot3d(m, interpolation_type, **kwds)
    G._set_extra_kwds(kwds)
    return G


def list_plot3d_tuples(v, interpolation_type, **kwds):
    r"""
    A 3-dimensional plot of a surface defined by the list `v`
    of points in 3-dimensional space.

    INPUT:

    - ``v`` -- something that defines a set of points in 3
      space, for example:

      - a matrix

        This will be if using an ``interpolation_type`` other than
        ``'linear'``, or if using ``num_points`` with ``'linear'``;
        otherwise see :func:`list_plot3d_matrix`.

      - a list of 3-tuples

      - a list of lists (all of the same length, under same conditions
        as a matrix)

    OPTIONAL KEYWORDS:

    - ``interpolation_type`` -- one of ``'linear'``, ``'clough'``
      (CloughTocher2D), ``'spline'``

      ``'linear'`` will perform linear interpolation

      The option 'clough' will interpolate by using a piecewise cubic
      interpolating Bezier polynomial on each triangle, using a
      Clough-Tocher scheme.  The interpolant is guaranteed to be
      continuously differentiable.

      The option ``'spline'`` interpolates using a bivariate B-spline.

      When ``v`` is a matrix the default is to use linear interpolation, when
      ``v`` is a list of points the default is ``'clough'``.

    - ``degree`` -- integer between 1 and 5, controls the degree of spline
      used for spline interpolation. For data that is highly oscillatory
      use higher values

    - ``point_list`` -- if ``point_list=True`` is passed, then if the array
      is a list of lists of length three, it will be treated as an
      array of points rather than a `3\times n` array.

    - ``num_points`` -- number of points to sample interpolating
      function in each direction.  By default for an `n\times n`
      array this is `n`.

    - ``**kwds`` -- all other arguments are passed to the
      surface function

    OUTPUT: a 3d plot

    EXAMPLES:

    All of these use this function; see :func:`list_plot3d` for other
    list plots::

        sage: from math import pi
        sage: m = matrix(RDF, 6, [sin(i^2 + j^2)
        ....:                     for i in [0,pi/5,..,pi] for j in [0,pi/5,..,pi]])
        sage: list_plot3d(m, color='yellow', interpolation_type='linear',  # indirect doctest
        ....:             num_points=5)
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='linear', num_points=5))

    ::

        sage: list_plot3d(m, color='yellow', interpolation_type='spline',
        ....:             frame_aspect_ratio=[1, 1, 1/3])
        Graphics3d Object

    .. PLOT::

        import numpy as np
        pi = float(pi)
        m = matrix(RDF, 6, [sin(i**2 + j**2) for i in np.linspace(0,pi,6) for j in np.linspace(0,pi,6)])
        sphinx_plot(list_plot3d(m, color='yellow', interpolation_type='spline', frame_aspect_ratio=[1, 1, 1/3]))

    ::

        sage: show(list_plot3d([[1, 1, 1], [1, 2, 1], [0, 1, 3], [1, 0, 4]],
        ....:                  point_list=True))

    .. PLOT::

        sphinx_plot(list_plot3d([[1, 1, 1], [1, 2, 1], [0, 1, 3], [1, 0, 4]],
                                point_list=True))

    ::

        sage: list_plot3d([(1, 2, 3), (0, 1, 3), (2, 1, 4), (1, 0, -2)],  # long time
        ....:             color='yellow', num_points=50)
        Graphics3d Object

    .. PLOT::

        sphinx_plot(list_plot3d([(1, 2, 3), (0, 1, 3), (2, 1, 4), (1, 0, -2)], color='yellow', num_points=50))
    """
    from matplotlib import tri
    import numpy
    from random import random
    from scipy import interpolate
    from .plot3d import plot3d

    if len(v) < 3:
        raise ValueError("we need at least 3 points to perform the "
                         "interpolation")

    x = [float(p[0]) for p in v]
    y = [float(p[1]) for p in v]
    z = [float(p[2]) for p in v]

    # If the (x,y)-coordinates lie in a one-dimensional subspace, the
    # matplotlib Delaunay code segfaults.  Therefore, we compute the
    # correlation of the x- and y-coordinates and add small random
    # noise to avoid the problem if needed.
    corr_matrix = numpy.corrcoef(x, y)
    if not (-0.9 <= corr_matrix[0, 1] <= 0.9):
        ep = .000001
        x = [float(p[0]) + random() * ep for p in v]
        y = [float(p[1]) + random() * ep for p in v]

    # If the list of data points has two points with the exact same
    # (x,y)-coordinate but different z-coordinates, then we sometimes
    # get segfaults.  The following block checks for this and raises
    # an exception if this is the case.
    # We also remove duplicate points (which matplotlib can't handle).
    # Alternatively, the code in the if block above which adds random
    # error could be applied to perturb the points.
    drop_list = []
    nb_points = len(x)
    for i in range(nb_points):
        for j in range(i + 1, nb_points):
            if x[i] == x[j] and y[i] == y[j]:
                if z[i] != z[j]:
                    raise ValueError("points with same x,y coordinates"
                                     " and different z coordinates were"
                                     " given. Interpolation cannot handle this.")
                elif z[i] == z[j]:
                    drop_list.append(j)
    x = [x[i] for i in range(nb_points) if i not in drop_list]
    y = [y[i] for i in range(nb_points) if i not in drop_list]
    z = [z[i] for i in range(nb_points) if i not in drop_list]

    xmin = float(min(x))
    xmax = float(max(x))
    ymin = float(min(y))
    ymax = float(max(y))

    num_points = kwds.get('num_points', int(4 * numpy.sqrt(len(x))))
    # arbitrary choice - assuming more or less a n x n grid of points
    # x should have n^2 entries. We sample 4 times that many points.

    if interpolation_type == 'linear':
        T = tri.Triangulation(x, y)
        f = tri.LinearTriInterpolator(T, z)
        j = complex(0, 1)
        from .parametric_surface import ParametricSurface

        def g(x, y):
            z = f(x, y)
            return (x, y, z)

        G = ParametricSurface(g, (list(numpy.r_[xmin:xmax:num_points * j]),
                                  list(numpy.r_[ymin:ymax:num_points * j])),
                              **kwds)
        G._set_extra_kwds(kwds)
        return G

    if interpolation_type == 'clough' or interpolation_type == 'default':

        points = [[x[i], y[i]] for i in range(len(x))]
        j = complex(0, 1)
        f = interpolate.CloughTocher2DInterpolator(points, z)
        from .parametric_surface import ParametricSurface

        def g(x, y):
            z = f([x, y]).item()
            return (x, y, z)

        G = ParametricSurface(g, (list(numpy.r_[xmin:xmax:num_points * j]),
                                  list(numpy.r_[ymin:ymax:num_points * j])),
                              **kwds)
        G._set_extra_kwds(kwds)
        return G

    if interpolation_type == 'spline':
        kx = kwds.get('kx', 3)
        ky = kwds.get('ky', 3)
        if 'degree' in kwds:
            kx = kwds['degree']
            ky = kwds['degree']
        s = kwds.get('smoothing', len(x) - numpy.sqrt(2 * len(x)))
        s = interpolate.bisplrep(x, y, z, [1] * len(x), xmin, xmax,
                                 ymin, ymax, kx=kx, ky=ky, s=s)

        def f(x, y):
            return interpolate.bisplev(x, y, s)

        return plot3d(f, (xmin, xmax), (ymin, ymax),
                      plot_points=[num_points, num_points], **kwds)



r"""
Iterator for Weil polynomials.

For `q` a prime power, a `q`-Weil polynomial is a monic polynomial with integer
coefficients whose complex roots all have absolute value `sqrt(q)`. The class
WeilPolynomials provides an iterable over a space of polynomials of this type;
it is possible to relax the monic condition by specifying one (or more) leading
coefficients. One may also impose certain congruence conditions; this can be
used to limit the Newton polygons of the resulting polynomials, or to lift
a polynomial specified by a congruence to a Weil polynomial.

For large jobs, one can set parallel=True to use OpenMP (if support was
enabled at compile time). Due to increased overhead, this is not recommended
for smaller problem sizes. To enable support, ensure that your compiler supports
OpenMP and remove the appropriate # characters in the distutils commands below.
(You may also need to move those lines to the start of the file.)

AUTHOR:
  -- Kiran S. Kedlaya (2007-05-28): initial version
  -- (2015-08-29): switch from NTL to FLINT
  -- (2017-10-03): consolidate Sage layer into .pyx file
                   define WeilPolynomials iterator
                   reverse convention for polynomials
                   pass multiprecision integers to/from C
  -- (2019-02-02): update for Python3
                   improve parallel mode
  -- (2019-12-19): final packaging for Sage (with help from David Roe)

A standalone version of this code can be found at
   https://github.com/kedlaya/root-unitary
"""
# Remove second # from the next two lines to enable OpenMP support.
##distutils: libraries = gomp
##distutils: extra_compile_args = -fopenmp

# ***************************************************************************
#       Copyright (C) 2019 Kiran S. Kedlaya <kskedl@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ***************************************************************************

cimport cython
from cython.parallel import prange
from libc.stdlib cimport malloc, free
from cysignals.signals cimport sig_on, sig_off

from sage.rings.rational_field import QQ
from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
from sage.functions.generalized import sgn

from sage.rings.integer cimport Integer
from sage.libs.gmp.types cimport mpz_t
from sage.libs.gmp.mpz cimport mpz_set
from sage.libs.flint.fmpz cimport *
from sage.libs.flint.fmpz_vec cimport *

cdef extern from "sage/rings/polynomial/weil/power_sums.c":
    ctypedef struct ps_static_data_t:
        pass

    ctypedef struct ps_dynamic_data_t:
        int flag
        # State of the iterator (0 = inactive, 1 = running,
        #                        2 = found a solution,
        #                       -1 = too many nodes)

        long node_count  # Number of terminal nodes encountered
        fmpz *sympol     # Return value (a polynomial)

    int has_openmp()
    ps_static_data_t *ps_static_init(int d, fmpz_t q, int coeffsign, fmpz_t lead,
                                     int cofactor, fmpz *modlist, long node_limit,
                                     int force_squarefree)
    ps_dynamic_data_t *ps_dynamic_init(int d, fmpz_t q, fmpz *coefflist)
    void ps_dynamic_split(ps_dynamic_data_t *dy_data, ps_dynamic_data_t *dy_data2) nogil
    void ps_static_clear(ps_static_data_t *st_data)
    void ps_dynamic_clear(ps_dynamic_data_t *dy_data)
    void next_pol(ps_static_data_t *st_data, ps_dynamic_data_t *dy_data, int max_steps) nogil

cdef class dfs_manager:
    """
    Data structure to manage depth-first search.

    Such a structure is created and managed by an instance of ``WeilPolynomials_iter``.
    There is generally no need for a user to manipulate it directly.
    """
    cdef int d
    cdef int num_processes
    cdef long node_limit
    cdef ps_static_data_t *ps_st_data
    cdef ps_dynamic_data_t **dy_data_buf

    def __cinit__(self, int d, q, coefflist, modlist, int sign, int cofactor,
                  long node_limit, int parallel, int force_squarefree):
        """
        Perform required C-level initialization (e.g., memory allocation).

        This is called automatically at object creation. It should
        never be called directly.

        TESTS:

        For some reason, Sage requires a dummy doctest to meet coverage requirements::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: it.process is not None # Verify object creation
            True
        """
        cdef fmpz_t temp_lead
        cdef fmpz_t temp_q
        cdef fmpz *temp_array
        cdef int i = 101 if parallel else 1

        self.d = d
        self.num_processes = i
        self.dy_data_buf = <ps_dynamic_data_t **>malloc(i*cython.sizeof(cython.pointer(ps_dynamic_data_t)))
        self.node_limit = node_limit
        fmpz_init(temp_lead)
        fmpz_set_mpz(temp_lead, Integer(coefflist[-1]).value)
        fmpz_init(temp_q)
        fmpz_set_mpz(temp_q, Integer(q).value)
        temp_array = _fmpz_vec_init(d + 1)
        for i in range(d + 1):
            fmpz_set_mpz(temp_array + i, Integer(modlist[i]).value)
        self.ps_st_data = ps_static_init(d, temp_q, sign, temp_lead, cofactor,
                                         temp_array, node_limit, force_squarefree)

        # Initialize processes, but assign work to only one process.
        # In parallel mode, other processes will get initialized later via work-stealing.
        for i in range(d+1):
            fmpz_set_mpz(temp_array+i, Integer(coefflist[i]).value)
        self.dy_data_buf[0] = ps_dynamic_init(d, temp_q, temp_array)
        for i in range(1, self.num_processes):
            self.dy_data_buf[i] = ps_dynamic_init(d, temp_q, NULL)

        fmpz_clear(temp_lead)
        fmpz_clear(temp_q)
        _fmpz_vec_clear(temp_array, d + 1)

    def __dealloc__(self):
        """
        Deallocate memory.
        """
        ps_static_clear(self.ps_st_data)
        self.ps_st_data = NULL
        if self.dy_data_buf != NULL:
            for i in range(self.num_processes):
                ps_dynamic_clear(self.dy_data_buf[i])
            free(self.dy_data_buf)
            self.dy_data_buf = NULL

    cpdef long node_count(self) noexcept:
        """
        Count nodes.

        This method should not be called directly. Instead, use the
        ``node_count`` method of an instance of ``WeilPolynomials`` or
        ``WeilPolynomials_iter``.

        TESTS::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: _ = next(it)
            sage: it.process.node_count()
            158
        """
        cdef long count = 0
        cdef int i
        for i in range(self.num_processes):
            count += self.dy_data_buf[i].node_count
        return count

    cpdef object advance_exhaust(self):
        """
        Advance the tree exhaustion.

        This method should not be called directly. Instead, use the iterator
        ``WeilPolynomials_iter`` or the iterable ``WeilPolynomials``.

        TESTS::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: it.process.advance_exhaust()[0]
            [3, 1, 1, -5, 1, -2, 1, -5, 1, 1, 3, 0, 0]
        """
        cdef int i, j, k, d = self.d, t = 1, u = 0, np = self.num_processes, max_steps = 1000
        cdef long ans_count = 0, ans_max = 10000
        cdef mpz_t z
        cdef Integer temp
        ans = []

        k = 1
        while (t and not u and ans_count < ans_max):
            if np == 1:  # Serial mode
                next_pol(self.ps_st_data, self.dy_data_buf[0], max_steps)
                t = self.dy_data_buf[0].flag
            else:  # Parallel mode
                t = 0
                k = (k<<1) % np  # Note that 2 is a primitive root mod np.
                with nogil:
                    sig_on()
                    for i in prange(np, schedule='dynamic'):  # Step each process forward
                        next_pol(self.ps_st_data, self.dy_data_buf[i], max_steps)
                        if self.dy_data_buf[i].flag:
                            t += 1
                        if self.dy_data_buf[i].flag == -1:
                            u += 1
                    for i in prange(np, schedule='dynamic'):  # Redistribute work to idle processes
                        j = (i-k) % np
                        ps_dynamic_split(self.dy_data_buf[j], self.dy_data_buf[i])
                    sig_off()
            for i in range(np):
                if self.dy_data_buf[i].flag == 2:  # Extract a solution
                    l = []
                    # Convert a vector of fmpz's into mpz's, then Integers.
                    for j in range(2 * d + 3):
                        flint_mpz_init_set_readonly(z, &self.dy_data_buf[i].sympol[j])
                        temp = Integer()
                        mpz_set(temp.value, z)
                        l.append(temp)
                        flint_mpz_clear_readonly(z)
                    ans.append(l)
                    ans_count += 1
        if u:
            print(u)
            raise RuntimeError("Node limit ({0:%d}) exceeded".format(self.node_limit))
        return ans


class WeilPolynomials_iter():
    r"""
    Iterator created by WeilPolynomials.

    EXAMPLES::

        sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
        sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
        sage: it = iter(w)
        sage: next(it)
        3*x^10 + x^9 + x^8 + 7*x^7 + 5*x^6 + 2*x^5 + 5*x^4 + 7*x^3 + x^2 + x + 3
        sage: w = WeilPolynomials(10,1,sign=-1,lead=[3,1,1])
        sage: it = iter(w)
        sage: next(it)
        3*x^10 + x^9 + x^8 + 6*x^7 - 2*x^6 + 2*x^4 - 6*x^3 - x^2 - x - 3
    """
    def __init__(self, d, q, sign, lead, node_limit,
                 parallel, squarefree, polring=None):
        r"""
        Create an iterator for Weil polynomials.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: next(it)
            3*x^10 + x^9 + x^8 + 7*x^7 + 5*x^6 + 2*x^5 + 5*x^4 + 7*x^3 + x^2 + x + 3
        """
        if polring is None:
            polring = PolynomialRing(QQ, name='x')
        self.pol = polring
        x = self.pol.gen()
        d = Integer(d)
        if sign != 1 and sign != -1:
            raise ValueError("invalid sign")
        if not q.is_integer() or q <= 0:
            raise ValueError("q must be a positive integer")
        if d % 2 == 0:
            if sign == 1:
                d2 = d//2
                num_cofactor = 0
            else:
                d2 = d//2 - 1
                num_cofactor = 3
        else:
            if not q.is_square():
                raise ValueError("degree must be even if q is not a square")
            d2 = d//2
            if sign == 1:
                num_cofactor = 1
            else:
                num_cofactor = 2
        try:
            leadlist = list(lead)
        except TypeError:
            leadlist = [(lead, 0)]
        coefflist = []
        modlist = []
        for i in leadlist:
            try:
                (j, k) = i
            except TypeError:
                (j, k) = (i, 0)
            j = Integer(j)
            k = Integer(k)
            if len(modlist) == 0 and k != 0:
                raise ValueError("leading coefficient must be specified exactly")
            if modlist and ((k != 0 and modlist[-1] % k != 0) or (k == 0 and modlist[-1] != 0)):
                raise ValueError("invalid moduli")
            coefflist.append(j)
            modlist.append(k)
        # Remove cofactor from initial coefficients
        if num_cofactor == 1:  # cofactor x + sqrt(q)
            for i in range(1, len(coefflist)):
                coefflist[i] -= coefflist[i-1]*q.sqrt()
        elif num_cofactor == 2:  # cofactor x + sqrt(q)
            for i in range(1, len(coefflist)):
                coefflist[i] += coefflist[i-1]*q.sqrt()
        elif num_cofactor == 3:  # cofactor x^2 - q
            for i in range(2, len(coefflist)):
                coefflist[i] += coefflist[i-2]*q
        # Asymmetrize initial coefficients
        for i in range(len(coefflist)):
            for j in range(1, (len(coefflist)-i+1)//2):
                coefflist[i+2*j] -= (d2-i).binomial(j)*(q**j)*coefflist[i]
        for _ in range(d2+1-len(coefflist)):
            coefflist.append(0)
            modlist.append(1)
        coeffsign = sgn(coefflist[0])
        coefflist = [x*coeffsign for x in reversed(coefflist)]
        if node_limit is None:
            node_limit = -1
        force_squarefree = Integer(squarefree)
        self.process = None if d2<0 else dfs_manager(
            d2, q, coefflist, modlist, coeffsign, num_cofactor,
            node_limit, parallel, force_squarefree)
        self.q = q
        self.squarefree = squarefree
        self.ans = []

    def __iter__(self):
        r"""
        Return the iterator (i.e. ``self``).

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: it.__iter__() is it
            True
        """
        return self

    def __next__(self):
        r"""
        Step the iterator forward.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: next(it)
            3*x^10 + x^9 + x^8 + 7*x^7 + 5*x^6 + 2*x^5 + 5*x^4 + 7*x^3 + x^2 + x + 3
        """
        if self.process is None:
            raise StopIteration
        if not self.ans:
            self.ans = self.process.advance_exhaust()
            if not self.ans:
                self.count = self.process.node_count()
                self.process = None
                raise StopIteration
        return self.pol(self.ans.pop())

    def node_count(self):
        r"""
        Return the number of terminal nodes found in the tree, excluding
        actual solutions.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = iter(w)
            sage: l = list(it)
            sage: it.node_count()
            158
        """
        if self.process is None:
            return self.count
        return self.process.node_count()


class WeilPolynomials():
    r"""
    Iterable for Weil polynomials, i.e., integer polynomials with all complex
    roots having a particular absolute value.

    Such polynomials `f` satisfy a functional equation

    .. MATH::

        T^d f(q/T) = s q^{d/2} f(T)

    where `d` is the degree of `f`, `s` is a sign and `q^{1/2}` is the absolute value
    of the roots of `f`.

    If parallel is False, then the order of values is descending lexicographical
    (i.e., polynomials with the largest coefficients of largest degrees sort first).

    If parallel is True, then the order of values is not specified. (Beware that
    due to increased overhead, parallel execution may not yield a significant
    speedup for small problem sizes.)

    INPUT:

    - ``d`` -- integer; the degree of the polynomials

    - ``q`` -- integer; the square of the complex absolute value of the roots

    - ``sign`` -- integer (default: `1`); the sign `s` of the functional equation

    - ``lead`` -- integer (default: `1`); list of integers or pairs of integers

        These are constraints on the leading coefficients of the generated polynomials.
        If pairs `(a, b)` of integers are given, they are treated as a constraint
        of the form `\equiv a \pmod{b}`; the moduli must be in decreasing order by
        divisibility, and the modulus of the leading coefficient must be 0.

    - ``node_limit`` -- integer (default: ``None``)

        If set, imposes an upper bound on the number of terminal nodes during the search
        (will raise a :exc:`RuntimeError` if exceeded).

    - ``parallel`` -- boolean (default: ``False``); whether to use multiple processes

        If set, will raise an error unless this file was compiled with OpenMP support
        (see instructions at the top of :mod:`sage.rings.polynomial.weil.weil_polynomials`).

    - ``squarefree`` -- boolean (default: ``False``)

        If set, only squarefree polynomials will be returned.

    - ``polring`` -- (optional) a polynomial ring in which to construct the results

    EXAMPLES:

    Some simple cases::

        sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
        sage: list(WeilPolynomials(2,2))
        [x^2 + 2*x + 2, x^2 + x + 2, x^2 + 2, x^2 - x + 2, x^2 - 2*x + 2]
        sage: l = list(WeilPolynomials(4,2))
        sage: l[0], l[-1]
        (x^4 + 4*x^3 + 8*x^2 + 8*x + 4, x^4 - 4*x^3 + 8*x^2 - 8*x + 4)
        sage: l = list(WeilPolynomials(3, 1, sign=-1))
        sage: l[0], l[-1]
        (x^3 + x^2 - x - 1, x^3 - 3*x^2 + 3*x - 1)

    By Kronecker's theorem, a monic integer polynomial has all roots of absolute
    value 1 if and only if it is a product of cyclotomic polynomials. For such a
    product to have positive sign of the functional equation, the factors `x-1`
    and `x+1` must each occur with even multiplicity. This code confirms
    Kronecker's theorem for polynomials of degree 6::

        sage: P.<x> = PolynomialRing(ZZ)
        sage: d = 6
        sage: ans1 = list(WeilPolynomials(d, 1, 1))
        sage: ans1.sort()
        sage: l = [(x-1)^2, (x+1)^2] + [cyclotomic_polynomial(n,x)
        ....:     for n in range(3, 2*d*d) if euler_phi(n) <= d]
        sage: w = WeightedIntegerVectors(d, [i.degree() for i in l])
        sage: ans2 = [prod(l[i]^v[i] for i in range(len(l))) for v in w]
        sage: ans2.sort()
        sage: print(ans1 == ans2)
        True

    Generating Weil polynomials with prescribed initial coefficients::

        sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
        sage: it = iter(w)
        sage: next(it)
        3*x^10 + x^9 + x^8 + 7*x^7 + 5*x^6 + 2*x^5 + 5*x^4 + 7*x^3 + x^2 + x + 3
        sage: w = WeilPolynomials(10,1,sign=-1,lead=[3,1,1])
        sage: it = iter(w)
        sage: next(it)
        3*x^10 + x^9 + x^8 + 6*x^7 - 2*x^6 + 2*x^4 - 6*x^3 - x^2 - x - 3

    TESTS:

    Test restriction of initial coefficients::

        sage: w1 = WeilPolynomials(10,1,sign=1,lead=3)
        sage: l1 = list(w1)
        sage: w2 = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
        sage: l2 = list(w2)
        sage: l3 = [i for i in l1 if i[1] == 1 and i[2] == 1]
        sage: l2 == l3
        True

        sage: w = WeilPolynomials(4,2,lead=[(1,0),(0,2)])
        sage: l = list(w)
        sage: l[0], l[-1]
        (x^4 + 4*x^3 + 8*x^2 + 8*x + 4, x^4 - 4*x^3 + 8*x^2 - 8*x + 4)
        sage: sorted(list(set(i[3] for i in l)))
        [-4, -2, 0, 2, 4]

    Test restriction to squarefree polynomials::

        sage: for (d,q,sign) in ((6,2,1),(6,4,-1),(5,4,-1)):
        ....:     w1 = WeilPolynomials(d,q,sign=sign)
        ....:     l1 = list(w1)
        ....:     w2 = WeilPolynomials(d,q,sign=sign,squarefree=True)
        ....:     l2 = list(w2)
        ....:     l3 = [i for i in l1 if i.is_squarefree()]
        ....:     print(l2 == l3)
        True
        True
        True

    Test that :issue:`29475` is resolved::

        sage: P.<x> = QQ[]
        sage: u = x^6 + x^5 + 6*x^4 - 2*x^3 + 66*x^2 + 121*x + 1331
        sage: u.is_weil_polynomial()
        True
        sage: u in WeilPolynomials(6, 11, 1, [1,1,6])
        True
        sage: u in WeilPolynomials(6, 11, 1, [(1,0),(1,11),(6,11)])
        True

    Test that :issue:`31809` is resolved::

        sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
        sage: foo = list(WeilPolynomials(12, 3, lead=(1,0,9,2,46), squarefree=False))
        sage: bar = list(WeilPolynomials(12, 3, lead=(1,0,9,2,46), squarefree=True))
        sage: bar == [f for f in foo if f.is_squarefree()]
        True

    Test that :issue:`32348` is resolved::

        sage: list(WeilPolynomials(10, 2, lead=(1,-3,5,-5,5,-5)))
        [x^10 - 3*x^9 + 5*x^8 - 5*x^7 + 5*x^6 - 5*x^5 + 10*x^4 - 20*x^3 + 40*x^2 - 48*x + 32]

    Test that :issue:`37860` is resolved::

        sage: list(WeilPolynomials(-1, 1))
        []
        sage: list(WeilPolynomials(0, 1, sign=-1))
        []
    """
    def __init__(self, d, q, sign=1, lead=1, node_limit=None,
                 parallel=False, squarefree=False, polring=None):
        r"""
        Initialize this iterable.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: w.__init__(10,1,sign=1,lead=[3,1,-1]) # Change parameters before iterating
            sage: it = iter(w)
            sage: next(it) # Results reflect the changed parameters
            3*x^10 + x^9 - x^8 + 7*x^7 + 5*x^6 - 2*x^5 + 5*x^4 + 7*x^3 - x^2 + x + 3
        """
        if parallel and not has_openmp():
            raise RuntimeError("Parallel execution not supported")
        self.data = (d, q, sign, lead, node_limit, parallel, squarefree, polring)

    def __iter__(self):
        r"""
        Construct the associated iterator.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: it = w.__iter__()
            sage: next(it)
            3*x^10 + x^9 + x^8 + 7*x^7 + 5*x^6 + 2*x^5 + 5*x^4 + 7*x^3 + x^2 + x + 3
        """
        w = WeilPolynomials_iter(*self.data)
        self.w = w
        return w

    def node_count(self):
        r"""
        Return the number of terminal nodes found in the tree, excluding actual solutions.

        EXAMPLES::

            sage: from sage.rings.polynomial.weil.weil_polynomials import WeilPolynomials
            sage: w = WeilPolynomials(10,1,sign=1,lead=[3,1,1])
            sage: l = list(w)
            sage: w.node_count()
            158
        """
        return self.w.node_count()
